@@ -26,13 +26,20 @@
 #define _XFDESKTOP_COMMON_H_
 
 #include <glib.h>
+#include <gdk/gdk.h>
+
+#include <X11/Xlib.h>
 
 #define BACKDROP_CHANNEL         "BACKDROP"
-#define DEFAULT_BACKDROP         DATADIR "/xfce4/backdrops/xfld.png"
+#define DEFAULT_BACKDROP         DATADIR "/xfce4/backdrops/xfce-smoke.png"
 #define LIST_TEXT                "# xfce backdrop list"
 #define XFDESKTOP_SELECTION_FMT  "XFDESKTOP_SELECTION_%d"
 #define XFDESKTOP_IMAGE_FILE_FMT "XFDESKTOP_IMAGE_FILE_%d"
 
+#define RELOAD_MESSAGE           "reload"
+#define MENU_MESSAGE             "menu"
+#define WINDOWLIST_MESSAGE       "windowlist"
+
 G_BEGIN_DECLS
 
 typedef enum
@@ -55,6 +62,10 @@ gchar **get_list_from_file(const gchar *);
 gboolean is_backdrop_list(const gchar *path);
 gboolean xfdesktop_check_image_file(const gchar *filename);
 
+void xfdesktop_send_client_message(Window xid, const gchar *msg);
+gboolean xfdesktop_check_is_running(Window *xid);
+gboolean xfdesktop_popup_grab_available(GdkWindow *win, guint32 timestamp);
+
 G_END_DECLS
 
 #endif
