@@ -69,6 +69,224 @@ static void delete_entry_cb (GtkWidget * widget, gpointer data);
 /* Main window */
 static void create_main_window (MenuEditor * me);
 
+/*******************************************************/
+/* Copy of g_markup_printf_escaped from the glib 2.4.8 */
+/*******************************************************/
+static const char *
+find_conversion (const char *format, const char **after)
+{
+  const char *start = format;
+  const char *cp;
+
+  while (*start != '\0' && *start != '%')
+    start++;
+
+  if (*start == '\0') {
+    *after = start;
+    return NULL;
+  }
+
+  cp = start + 1;
+
+  if (*cp == '\0') {
+    *after = cp;
+    return NULL;
+  }
+
+  /* Test for positional argument.  */
+  if (*cp >= '0' && *cp <= '9') {
+    const char *np;
+
+    for (np = cp; *np >= '0' && *np <= '9'; np++);
+    if (*np == '$')
+      cp = np + 1;
+  }
+
+  /* Skip the flags.  */
+  for (;;) {
+    if (*cp == '\'' || *cp == '-' || *cp == '+' || *cp == ' ' || *cp == '#' || *cp == '0')
+      cp++;
+    else
+      break;
+  }
+
+  /* Skip the field width.  */
+  if (*cp == '*') {
+    cp++;
+
+    /* Test for positional argument.  */
+    if (*cp >= '0' && *cp <= '9') {
+      const char *np;
+
+      for (np = cp; *np >= '0' && *np <= '9'; np++);
+      if (*np == '$')
+        cp = np + 1;
+    }
+  }
+  else {
+    for (; *cp >= '0' && *cp <= '9'; cp++);
+  }
+
+  /* Skip the precision.  */
+  if (*cp == '.') {
+    cp++;
+    if (*cp == '*') {
+      /* Test for positional argument.  */
+      if (*cp >= '0' && *cp <= '9') {
+        const char *np;
+
+        for (np = cp; *np >= '0' && *np <= '9'; np++);
+        if (*np == '$')
+          cp = np + 1;
+      }
+    }
+    else {
+      for (; *cp >= '0' && *cp <= '9'; cp++);
+    }
+  }
+
+  /* Skip argument type/size specifiers.  */
+  while (*cp == 'h' || *cp == 'L' || *cp == 'l' || *cp == 'j' || *cp == 'z' || *cp == 'Z' || *cp == 't')
+    cp++;
+
+  /* Skip the conversion character.  */
+  cp++;
+
+  *after = cp;
+  return start;
+}
+
+static char *
+menueditor_markup_vprintf_escaped (const char *format, va_list args)
+{
+  GString *format1;
+  GString *format2;
+  GString *result = NULL;
+  gchar *output1 = NULL;
+  gchar *output2 = NULL;
+  const char *p, *op1, *op2;
+  va_list args2;
+
+  /* The technique here, is that we make two format strings that
+   * have the identical conversions in the identical order to the
+   * original strings, but differ in the text in-between. We
+   * then use the normal g_strdup_vprintf() to format the arguments
+   * with the two new format strings. By comparing the results,
+   * we can figure out what segments of the output come from
+   * the the original format string, and what from the arguments,
+   * and thus know what portions of the string to escape.
+   *
+   * For instance, for:
+   *
+   *  menueditor_markup_printf_escaped ("%s ate %d apples", "Susan & Fred", 5);
+   *
+   * We form the two format strings "%sX%dX" and %sY%sY". The results
+   * of formatting with those two strings are
+   *
+   * "%sX%dX" => "Susan & FredX5X"
+   * "%sY%dY" => "Susan & FredY5Y"
+   *
+   * To find the span of the first argument, we find the first position
+   * where the two arguments differ, which tells us that the first
+   * argument formatted to "Susan & Fred". We then escape that
+   * to "Susan &amp; Fred" and join up with the intermediate portions
+   * of the format string and the second argument to get
+   * "Susan &amp; Fred ate 5 apples".
+   */
+
+  /* Create the two modified format strings
+   */
+  format1 = g_string_new (NULL);
+  format2 = g_string_new (NULL);
+  p = format;
+  while (TRUE) {
+    const char *after;
+    const char *conv = find_conversion (p, &after);
+    if (!conv)
+      break;
+
+    g_string_append_len (format1, conv, after - conv);
+    g_string_append_c (format1, 'X');
+    g_string_append_len (format2, conv, after - conv);
+    g_string_append_c (format2, 'Y');
+
+    p = after;
+  }
+
+  /* Use them to format the arguments
+   */
+  G_VA_COPY (args2, args);
+
+  output1 = g_strdup_vprintf (format1->str, args);
+  va_end (args);
+  if (!output1)
+    goto cleanup;
+
+  output2 = g_strdup_vprintf (format2->str, args2);
+  va_end (args2);
+  if (!output2)
+    goto cleanup;
+
+  result = g_string_new (NULL);
+
+  /* Iterate through the original format string again,
+   * copying the non-conversion portions and the escaped
+   * converted arguments to the output string.
+   */
+  op1 = output1;
+  op2 = output2;
+  p = format;
+  while (TRUE) {
+    const char *after;
+    const char *output_start;
+    const char *conv = find_conversion (p, &after);
+    char *escaped;
+
+    if (!conv) {                /* The end, after points to the trailing \0 */
+      g_string_append_len (result, p, after - p);
+      break;
+    }
+
+    g_string_append_len (result, p, conv - p);
+    output_start = op1;
+    while (*op1 == *op2) {
+      op1++;
+      op2++;
+    }
+
+    escaped = g_markup_escape_text (output_start, op1 - output_start);
+    g_string_append (result, escaped);
+    g_free (escaped);
+
+    p = after;
+    op1++;
+    op2++;
+  }
+
+cleanup:
+  g_string_free (format1, TRUE);
+  g_string_free (format2, TRUE);
+  g_free (output1);
+  g_free (output2);
+
+  if (result)
+    return g_string_free (result, FALSE);
+  else
+    return NULL;
+}
+
+char *
+menueditor_markup_printf_escaped (const char *format, ...)
+{
+  char *result;
+  va_list args;
+
+  va_start (args, format);
+  result = menueditor_markup_vprintf_escaped (format, args);
+  va_end (args);
+
+  return result;
+}
 
 /*****************************/
 /* Manage icon theme changes */
@@ -358,7 +576,7 @@ load_menu_in_tree (xmlNodePtr menu, GtkTreeIter * p, gpointer data)
 
     /* separator */
     if (!xmlStrcmp (menu->name, (xmlChar *) "separator")) {
-      name = g_strdup_printf (SEPARATOR_FORMAT, _("--- separator ---"));
+      name = menueditor_markup_printf_escaped (SEPARATOR_FORMAT, _("--- separator ---"));
 
       gtk_tree_store_append (me->treestore, &c, p);
       gtk_tree_store_set (me->treestore, &c,
@@ -370,8 +588,8 @@ load_menu_in_tree (xmlNodePtr menu, GtkTreeIter * p, gpointer data)
       prop_name = xmlGetProp (menu, "name");
       prop_cmd = xmlGetProp (menu, "cmd");
 
-      name = g_strdup_printf (NAME_FORMAT, prop_name);
-      cmd = g_strdup_printf (COMMAND_FORMAT, prop_cmd);
+      name = menueditor_markup_printf_escaped (NAME_FORMAT, prop_name);
+      cmd = menueditor_markup_printf_escaped (COMMAND_FORMAT, prop_cmd);
 
       gtk_tree_store_append (me->treestore, &c, p);
       gtk_tree_store_set (me->treestore, &c,
@@ -383,7 +601,7 @@ load_menu_in_tree (xmlNodePtr menu, GtkTreeIter * p, gpointer data)
     if (!xmlStrcmp (menu->name, (xmlChar *) "menu")) {
       prop_name = xmlGetProp (menu, "name");
 
-      name = g_strdup_printf (MENU_FORMAT, prop_name);
+      name = menueditor_markup_printf_escaped (MENU_FORMAT, prop_name);
 
       gtk_tree_store_append (me->treestore, &c, p);
       gtk_tree_store_set (me->treestore, &c,
@@ -397,12 +615,12 @@ load_menu_in_tree (xmlNodePtr menu, GtkTreeIter * p, gpointer data)
       prop_type = xmlGetProp (menu, "type");
       prop_src = xmlGetProp (menu, "src");
 
-      name = g_strdup_printf (INCLUDE_FORMAT, _("--- include ---"));
+      name = menueditor_markup_printf_escaped (INCLUDE_FORMAT, _("--- include ---"));
 
       if (!xmlStrcmp (prop_type, (xmlChar *) "system"))
-        src = g_strdup_printf (INCLUDE_PATH_FORMAT, _("system"));
+        src = menueditor_markup_printf_escaped (INCLUDE_PATH_FORMAT, _("system"));
       else
-        src = g_strdup_printf (INCLUDE_PATH_FORMAT, prop_src);
+        src = menueditor_markup_printf_escaped (INCLUDE_PATH_FORMAT, prop_src);
 
       gtk_tree_store_append (me->treestore, &c, p);
       gtk_tree_store_set (me->treestore, &c,
@@ -415,8 +633,8 @@ load_menu_in_tree (xmlNodePtr menu, GtkTreeIter * p, gpointer data)
       prop_name = xmlGetProp (menu, "name");
       prop_cmd = xmlGetProp (menu, "cmd");
 
-      name = g_strdup_printf (NAME_FORMAT, prop_name);
-      cmd = g_strdup_printf (COMMAND_FORMAT, prop_cmd);
+      name = menueditor_markup_printf_escaped (NAME_FORMAT, prop_name);
+      cmd = menueditor_markup_printf_escaped (COMMAND_FORMAT, prop_cmd);
 
       gtk_tree_store_append (me->treestore, &c, p);
       gtk_tree_store_set (me->treestore, &c,
@@ -427,7 +645,7 @@ load_menu_in_tree (xmlNodePtr menu, GtkTreeIter * p, gpointer data)
     if (!xmlStrcmp (menu->name, (xmlChar *) "title")) {
       prop_name = xmlGetProp (menu, "name");
 
-      title = g_strdup_printf (TITLE_FORMAT, prop_name);
+      title = menueditor_markup_printf_escaped (TITLE_FORMAT, prop_name);
 
       gtk_tree_store_append (me->treestore, &c, p);
       gtk_tree_store_set (me->treestore, &c,
@@ -798,9 +1016,6 @@ treeview_button_pressed_cb (GtkTreeView * treeview, GdkEventButton * event, gpoi
       gtk_tree_model_get_value (GTK_TREE_MODEL (me->treestore), &iter, POINTER_COLUMN, &val);
       node = g_value_get_pointer (&val);
 
-      if (!xmlStrcmp (node->name, "separator"))
-        return TRUE;
-
       /* Create the popup menu */
       popup_menu = gtk_menu_new ();
 
@@ -828,6 +1043,9 @@ treeview_button_pressed_cb (GtkTreeView * treeview, GdkEventButton * event, gpoi
       g_signal_connect ((gpointer) moveup_menuitem, "activate", G_CALLBACK (entry_up_cb), me);
       g_signal_connect ((gpointer) movedown_menuitem, "activate", G_CALLBACK (entry_down_cb), me);
 
+      if (!xmlStrcmp (node->name, "separator"))
+	gtk_widget_set_sensitive (edit_menuitem, FALSE);
+
       gtk_widget_show_all (popup_menu);
       gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time ());
       return TRUE;
@@ -967,8 +1185,9 @@ create_main_window (MenuEditor * me)
 
   /* DND */
   GtkTargetEntry gte[] = { {"XFCE_MENU_ENTRY", GTK_TARGET_SAME_WIDGET, 0},
-			   {"text/plain", 0, 1},
-			   {"application/x-desktop", 0, 2} };
+  {"text/plain", 0, 1},
+  {"application/x-desktop", 0, 2}
+  };
 
   me->accel_group = gtk_accel_group_new ();
   me->icon_theme = xfce_icon_theme_get_for_screen (NULL);
@@ -1320,23 +1539,50 @@ open_menu_file (gchar * menu_file, MenuEditor * me)
 /***********************************************************/
 /* Get the menu file in use (inspired from xfdesktop code) */
 /***********************************************************/
-/* FIXME: move to common/ */
 static gchar *
 get_default_menu_file ()
 {
-  gchar filename[PATH_MAX];
-  const gchar *env = g_getenv ("XFCE_DISABLE_USER_CONFIG");
+  XfceKiosk *kiosk;
+  gboolean user_menu;
+  gchar filename[PATH_MAX], searchpath[PATH_MAX * 3 + 2], **all_dirs;
+  gint i;
 
-  if (!env || !strcmp (env, "0")) {
-    gchar *usermenu = xfce_resource_lookup (XFCE_RESOURCE_CONFIG,
-                                            "xfce4/desktop/menu.xml");
-    if (g_file_test (usermenu, G_FILE_TEST_IS_REGULAR))
-      return usermenu;
-    g_free (usermenu);
+  kiosk = xfce_kiosk_new ("xfdesktop");
+  user_menu = xfce_kiosk_query (kiosk, "UserMenu");
+  xfce_kiosk_free (kiosk);
+
+  if (!user_menu) {
+    const gchar *userhome = xfce_get_homedir ();
+    all_dirs = xfce_resource_lookup_all (XFCE_RESOURCE_CONFIG, "xfce4/desktop/");
+
+    for (i = 0; all_dirs[i]; i++) {
+      if (strstr (all_dirs[i], userhome) != all_dirs[i]) {
+        g_snprintf (searchpath, PATH_MAX * 3 + 2, "%s%%F.%%L:%s%%F.%%l:%s%%F", all_dirs[i], all_dirs[i], all_dirs[i]);
+        if (xfce_get_path_localized (filename, PATH_MAX, searchpath, "menu.xml", G_FILE_TEST_IS_REGULAR)) {
+          g_strfreev (all_dirs);
+          return g_strdup (filename);
+        }
+      }
+    }
+    g_strfreev (all_dirs);
   }
+  else {
+    gchar *menu_file = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
+                                                    "xfce4/desktop/menu.xml", FALSE);
+    if (menu_file && g_file_test (menu_file, G_FILE_TEST_IS_REGULAR))
+      return menu_file;
+    else if (menu_file)
+      g_free (menu_file);
 
-  if (xfce_get_path_localized (filename, PATH_MAX, SEARCHPATH, "menu.xml", G_FILE_TEST_IS_REGULAR)) {
-    return g_strdup (filename);
+    all_dirs = xfce_resource_lookup_all (XFCE_RESOURCE_CONFIG, "xfce4/desktop/");
+    for (i = 0; all_dirs[i]; i++) {
+      g_snprintf (searchpath, PATH_MAX * 3 + 2, "%s%%F.%%L:%s%%F.%%l:%s%%F", all_dirs[i], all_dirs[i], all_dirs[i]);
+      if (xfce_get_path_localized (filename, PATH_MAX, searchpath, "menu.xml", G_FILE_TEST_IS_REGULAR)) {
+        g_strfreev (all_dirs);
+        return g_strdup (filename);
+      }
+    }
+    g_strfreev (all_dirs);
   }
 
   g_warning ("%s: Could not locate a menu definition file", PACKAGE);
