@@ -60,7 +60,7 @@
 #define CATEGORIES_FILE "xfce-registered-categories.xml"
 
 static void menu_dentry_legacy_init();
-G_INLINE_FUNC gboolean menu_dentry_legacy_need_update(XfceDesktopMenu *desktop_menu);
+gboolean menu_dentry_legacy_need_update(XfceDesktopMenu *desktop_menu);
 static void menu_dentry_legacy_add_all(XfceDesktopMenu *desktop_menu,
 		MenuPathType pathtype);
 
@@ -87,7 +87,7 @@ static gchar **legacy_dirs = NULL;
 static GHashTable *dir_to_cat = NULL;
 
 /* we don't want most command-line parameters if they're given. */
-G_INLINE_FUNC gchar *
+gchar *
 _sanitise_dentry_cmd(gchar *cmd)
 {
 	gchar *p;
@@ -101,7 +101,7 @@ _sanitise_dentry_cmd(gchar *cmd)
 	return cmd;
 }
 
-G_INLINE_FUNC gint
+gint
 _get_path_depth(const gchar *path)
 {
 	gchar *p;
@@ -253,7 +253,7 @@ _ensure_path(XfceDesktopMenu *desktop_menu, const gchar *path)
 	return submenu;
 }
 
-static void
+static gboolean
 menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 		MenuPathType pathtype, gboolean is_legacy, const gchar *extra_cat)
 {
@@ -265,6 +265,7 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 	GPtrArray *newpaths = NULL;
 	const gchar *name;
 	gchar tmppath[2048];
+	gboolean ret = FALSE;
 
 	xfce_desktop_entry_get_string (de, "OnlyShowIn", FALSE, &onlyshowin);
 	/* each element needs to be ';'-terminated.  i'm not working around
@@ -286,6 +287,19 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 		goto cleanup;
 	if((p = strchr(exec, ' ')))
 		*p = 0;
+    /* filter out quotes around the command (yeah, people do that!) */
+    if (exec[0] == '"') {
+        int i;
+
+        for (i = 1; exec[i-1] != '\0'; ++i) {
+            if (exec[i] != '"')
+                exec[i-1] = exec[i];
+            else {
+                exec[i-1] = '\0';
+                break;
+            }
+        }
+    }
 	if(blacklist && g_hash_table_lookup(blacklist, exec))
 		goto cleanup;
 	p = g_find_program_in_path(exec);
@@ -295,9 +309,9 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 
 	xfce_desktop_entry_get_string (de, "Categories", TRUE, &categories);
 	
-	if(categories) {
+	if(categories || !is_legacy) {
 		/* hack: leave out items that look like they are KDE control panels */
-		if(strstr(categories, ";X-KDE-"))
+		if(categories && strstr(categories, ";X-KDE-"))
 			goto cleanup;
 		
 		if(pathtype==MPATH_SIMPLE || pathtype==MPATH_SIMPLE_UNIQUE)
@@ -310,8 +324,7 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 	} else if(is_legacy) {
 		newpaths = g_ptr_array_new();
 		g_ptr_array_add(newpaths, g_strdup(extra_cat));
-	} else
-		goto cleanup;
+	}
 	
 	if(pathtype == MPATH_SIMPLE_UNIQUE) {
 		/* grab first of the most general */
@@ -349,6 +362,7 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 				xfce_app_menu_item_get_needs_term(XFCE_APP_MENU_ITEM(mi)),
 				xfce_app_menu_item_get_startup_notification(XFCE_APP_MENU_ITEM(mi)),
 				menu, menu_pos, NULL);
+		ret = TRUE;
 	} else if(pathtype == MPATH_MULTI_UNIQUE) {
 		/* grab most specific */
 		path = _build_path(desktop_menu->dentry_basepath,
@@ -383,6 +397,7 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 				xfce_app_menu_item_get_needs_term(XFCE_APP_MENU_ITEM(mi)),
 				xfce_app_menu_item_get_startup_notification(XFCE_APP_MENU_ITEM(mi)),
 				menu, menu_pos, NULL);
+		ret = TRUE;
 	} else {
 		if(pathtype == MPATH_MULTI)
 			_prune_generic_paths(newpaths);
@@ -421,6 +436,7 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 					xfce_app_menu_item_get_needs_term(XFCE_APP_MENU_ITEM(mi)),
 					xfce_app_menu_item_get_startup_notification(XFCE_APP_MENU_ITEM(mi)),
 					menu, menu_pos, NULL);
+			ret = TRUE;
 			g_free(path);
 		}
 		path = NULL;
@@ -442,9 +458,11 @@ menu_dentry_parse_dentry(XfceDesktopMenu *desktop_menu, XfceDesktopEntry *de,
 		g_free(exec);
 	if(path)
 		g_free(path);
+
+	return ret;
 }
 
-static void
+static gboolean
 menu_dentry_parse_dentry_file(XfceDesktopMenu *desktop_menu,
 		const gchar *filename, MenuPathType pathtype) 
 {
@@ -455,9 +473,13 @@ menu_dentry_parse_dentry_file(XfceDesktopMenu *desktop_menu,
 	dentry = xfce_desktop_entry_new(filename, dentry_keywords,
 			G_N_ELEMENTS(dentry_keywords));
 	if(dentry) {
-		menu_dentry_parse_dentry(desktop_menu, dentry, pathtype, FALSE, NULL);
+		gboolean ret = menu_dentry_parse_dentry(desktop_menu, dentry,
+				pathtype, FALSE, NULL);
 		g_object_unref(G_OBJECT(dentry));
+		return ret;
 	}
+	
+	return FALSE;
 }
 
 static gint
@@ -465,18 +487,26 @@ dentry_recurse_dir(GDir *dir, const gchar *path, XfceDesktopMenu *desktop_menu,
 		MenuPathType pathtype)
 {
 	const gchar *file;
-	gchar *fullpath;
+	gchar fullpath[PATH_MAX];
 	GDir *d1;
 	gint ndirs = 1;
 	struct stat st;
 	
 	while((file=g_dir_read_name(dir))) {
 		if(g_str_has_suffix(file, ".desktop")) {
-			fullpath = g_build_filename(path, file, NULL);
-			menu_dentry_parse_dentry_file(desktop_menu, fullpath, pathtype);
-			g_free(fullpath);
+			if(!g_hash_table_lookup(desktop_menu->menu_entry_hash, file)) {
+				g_snprintf(fullpath, PATH_MAX, "%s" G_DIR_SEPARATOR_S "%s",
+						path, file);
+				if(menu_dentry_parse_dentry_file(desktop_menu, fullpath,
+						pathtype))
+				{
+					g_hash_table_insert(desktop_menu->menu_entry_hash,
+							g_strdup(file), GINT_TO_POINTER(1));
+				}
+			}
 		} else {
-			fullpath = g_build_path(G_DIR_SEPARATOR_S, path, file, NULL);
+			g_snprintf(fullpath, PATH_MAX, "%s" G_DIR_SEPARATOR_S "%s",
+					path, file);
 			if((d1=g_dir_open(fullpath, 0, NULL))) {
 				if(!stat(fullpath, &st)) {
 					g_hash_table_insert(desktop_menu->dentrydir_mtimes,
@@ -485,7 +515,6 @@ dentry_recurse_dir(GDir *dir, const gchar *path, XfceDesktopMenu *desktop_menu,
 				ndirs += dentry_recurse_dir(d1, fullpath, desktop_menu, pathtype);
 				g_dir_close(d1);
 			}
-			g_free(fullpath);
 		}
 	}
 	
@@ -582,11 +611,6 @@ desktop_menu_dentry_parse_files(XfceDesktopMenu *desktop_menu,
 			g_hash_table_insert(blacklist, blacklist_arr[i], GINT_TO_POINTER(1));
 	}
 	
-	if(desktop_menu->dentrydir_mtimes)
-		g_hash_table_destroy(desktop_menu->dentrydir_mtimes);
-	desktop_menu->dentrydir_mtimes = g_hash_table_new_full(g_str_hash,
-			g_str_equal, (GDestroyNotify)g_free, NULL);
-	
 	/* lookup applications/ directories */
 	homepath = xfce_get_homefile(".local", "share", NULL);
 	if(kdedir) {
@@ -666,7 +690,7 @@ desktop_menu_dentry_need_update(XfceDesktopMenu *desktop_menu)
  * legacy dir support.  bleh.
  ******************************************************************************/
 
-static void
+static gboolean
 menu_dentry_legacy_parse_dentry_file(XfceDesktopMenu *desktop_menu,
 		const gchar *filename, const gchar *catdir, MenuPathType pathtype)
 {
@@ -688,9 +712,13 @@ menu_dentry_legacy_parse_dentry_file(XfceDesktopMenu *desktop_menu,
 	dentry = xfce_desktop_entry_new(filename, dentry_keywords,
 			G_N_ELEMENTS(dentry_keywords));
 	if(dentry) {
-		menu_dentry_parse_dentry(desktop_menu, dentry, pathtype, TRUE, category);
+		gboolean ret = menu_dentry_parse_dentry(desktop_menu, dentry,
+				pathtype, TRUE, category);
 		g_object_unref(G_OBJECT(dentry));
+		return ret;
 	}
+
+	return FALSE;
 }
 
 static void
@@ -720,8 +748,14 @@ menu_dentry_legacy_process_dir(XfceDesktopMenu *desktop_menu,
 					(catdir ? catdir : file), pathtype);
 		} else if(catdir && g_str_has_suffix(file, ".desktop")) {
 			/* we're also going to ignore category-less .desktop files. */
-			menu_dentry_legacy_parse_dentry_file(desktop_menu, fullpath,
-					catdir, pathtype);
+			if(!g_hash_table_lookup(desktop_menu->menu_entry_hash, file)) {
+				if(menu_dentry_legacy_parse_dentry_file(desktop_menu,
+						fullpath, catdir, pathtype))
+				{
+					g_hash_table_insert(desktop_menu->menu_entry_hash,
+							g_strdup(file), GINT_TO_POINTER(1));
+				}
+			}
 		}
 	}
 	
