@@ -82,8 +82,6 @@ enum {
 	MI_BUILTIN_QUIT = 1
 };
 
-G_MODULE_IMPORT void quit(gboolean force);
-
 struct MenuFileParserState {
 	gboolean started;
 	GQueue *branches;  /* GtkWidget * menus */
@@ -114,7 +112,20 @@ _do_builtin(GtkMenuItem *mi, gpointer user_data)
 	
 	switch(type) {
 		case MI_BUILTIN_QUIT:
-			quit(FALSE);
+			{
+				GModule *parent_exe = g_module_open(NULL, 0);
+				if(parent_exe) {
+					void (*builtin_quit)(gboolean) = NULL;
+					if(g_module_symbol(parent_exe, "quit",
+							(gpointer)&builtin_quit))
+					{
+						builtin_quit(FALSE);
+					} else
+						g_warning("XfceDesktopMenu: Unable to find 'quit' symbol in parent executable.");
+					g_module_close(parent_exe);
+				} else
+					g_warning("XfceDesktopMenu: Unable to dlopen() parent exe.");
+			}
 			break;
 		default:
 			g_warning("XfceDesktopMenu: unknown builtin type (%d)\n", type);
@@ -497,11 +508,18 @@ menu_file_xml_start(GMarkupParseContext *context, const gchar *element_name,
 							attribute_values[j], state->cur_branch,
 							state->cur_path, FALSE, FALSE);
 				} else {
-					gchar *menuincfile = xfce_get_userfile(attribute_values[j],
-							NULL);
-					desktop_menu_file_parse(state->desktop_menu, menuincfile,
-							state->cur_branch, state->cur_path, FALSE, FALSE);
-					g_free(menuincfile);
+					gchar tmp[PATH_MAX];
+					gchar *menuincfile;
+					
+					g_snprintf(tmp, PATH_MAX, "xfce4/desktop/%s",
+							attribute_values[j]);
+					menuincfile = xfce_resource_lookup(XFCE_RESOURCE_CONFIG, tmp);
+					if(menuincfile) {
+						desktop_menu_file_parse(state->desktop_menu,
+								menuincfile, state->cur_branch,
+								state->cur_path, FALSE, FALSE);
+						g_free(menuincfile);
+					}
 				}
 			}
 		} else if(!strcmp(attribute_values[i], "system")) {
@@ -511,7 +529,7 @@ menu_file_xml_start(GMarkupParseContext *context, const gchar *element_name,
 			k = _find_attribute(attribute_names, "unique");
 			l = _find_attribute(attribute_names, "legacy");
 			
-			if(k != -1 && !strcmp(attribute_values[i], "false"))
+			if(k != -1 && !strcmp(attribute_values[k], "false"))
 				only_unique = FALSE;				
 			if(l != -1 && !strcmp(attribute_values[l], "false"))
 				do_legacy = FALSE;
@@ -699,7 +717,7 @@ desktop_menu_file_need_update(XfceDesktopMenu *desktop_menu)
 	
 	g_return_val_if_fail(desktop_menu != NULL, FALSE);
 	
-	if(!desktop_menu->menu)
+	if(!desktop_menu->menu || !desktop_menu->menufile_mtimes)
 		return TRUE;
 	
 	desktop_menu->modified = FALSE;
