@@ -54,7 +54,7 @@
 #endif
 
 #include <glib.h>
-#include <libxfce4util/i18n.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include "desktop-menuspec.h"
 #include "desktop-menu-private.h"
