@@ -76,6 +76,8 @@ struct _XfceDesktopPriv
 {
 	GdkScreen *gscreen;
 	NetkScreen *netk_screen;
+    
+	GdkPixmap *bg_pixmap;
 	
 	guint nbackdrops;
 	XfceBackdrop **backdrops;
@@ -132,7 +134,7 @@ save_list_file_minus_one(const gchar *filename, const gchar **files, gint badi)
 	fclose(fp);
 }
 
-G_INLINE_FUNC gint
+gint
 count_elements(const gchar **list)
 {
 	gint i, c = 0;
@@ -247,13 +249,11 @@ static void
 backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 {
 	GtkWidget *desktop = user_data;
-	GtkStyle *style;
 	GdkPixbuf *pix;
 	GdkPixmap *pmap = NULL;
 	GdkColormap *cmap;
 	GdkScreen *gscreen;
 	GdkRectangle rect;
-	GdkEventExpose evt;
 	Pixmap xid;
 	GdkWindow *groot;
 	
@@ -284,7 +284,6 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 		 * probably still faster than redoing the whole thing. */
 		GdkPixmap *cur_pmap = NULL;
 		GdkPixbuf *cur_pbuf = NULL;
-		//GdkRectangle rect;
 		gint i, n = -1, swidth, sheight;
 		
 		for(i = 0; i < XFCE_DESKTOP(desktop)->priv->nbackdrops; i++) {
@@ -301,9 +300,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 		swidth = gdk_screen_get_width(gscreen);
 		sheight = gdk_screen_get_height(gscreen);
 		
-		style = gtk_widget_get_style(desktop);
-		if(style)
-			cur_pmap = style->bg_pixmap[GTK_STATE_NORMAL];
+		cur_pmap = XFCE_DESKTOP(desktop)->priv->bg_pixmap;
 		if(cur_pmap) {
 			gint pw, ph;
 			gdk_drawable_get_size(GDK_DRAWABLE(cur_pmap), &pw, &ph);
@@ -353,36 +350,13 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 	gdk_error_trap_pop();
 	
 	/* clear the old pixmap, if any */
-	style = gtk_widget_get_style(desktop);
-	if(style->bg_pixmap[GTK_STATE_NORMAL]) {
-		g_object_unref(G_OBJECT(style->bg_pixmap[GTK_STATE_NORMAL]));
-		style->bg_pixmap[GTK_STATE_NORMAL] = NULL;
-	}
-	gtk_widget_set_style(desktop, NULL);
+	if(XFCE_DESKTOP(desktop)->priv->bg_pixmap)
+		g_object_unref(G_OBJECT(XFCE_DESKTOP(desktop)->priv->bg_pixmap));
 	
-	/* create a new style, attach it to the window, and add the new pixmap */
-	style = gtk_style_new();
-	gtk_style_attach(style, desktop->window);
-	if(style->bg_pixmap[GTK_STATE_NORMAL]) {
-		/* if the style already has a BG pixmap set, ditch it */
-		g_object_unref(G_OBJECT(style->bg_pixmap[GTK_STATE_NORMAL]));
-	}
-	style->bg_pixmap[GTK_STATE_NORMAL] = pmap;
-	
-	/* set the widget's window style and queue it for drawing */
-	gtk_widget_set_style(desktop, style);
-	g_object_unref(G_OBJECT(style));
-	/* FIXME: all we should need to do is gtk_widget_queue_draw_area(), but that
-	 * isn't working for some reason */
-	/* gtk_widget_queue_draw_area(desktop, rect.x, rect.y, rect.width, rect.height); */
-	evt.type = GDK_EXPOSE;
-	evt.window = desktop->window;
-	evt.send_event = FALSE;
-	memcpy(&evt.area, &rect, sizeof(GdkRectangle));
-	evt.region = gdk_region_rectangle(&rect);
-	evt.count = 0;
-	gtk_widget_send_expose(desktop, (GdkEvent *)&evt);
-	gdk_region_destroy(evt.region);
+	/* set the new pixmap and tell gtk to redraw it */
+	XFCE_DESKTOP(desktop)->priv->bg_pixmap = pmap;
+	gdk_window_set_back_pixmap(desktop->window, pmap, FALSE);
+	gtk_widget_queue_draw_area(desktop, rect.x, rect.y, rect.width, rect.height);
 }
 
 static void
@@ -632,12 +606,35 @@ xfce_desktop_finalize(GObject *object)
 		desktop->priv->backdrops = NULL;
 	}
 	
+	if(desktop->priv->bg_pixmap)
+		g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
+	
 	g_free(desktop->priv);
 	desktop->priv = NULL;
 	
 	G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
+static gboolean
+desktop_expose_cb(GtkWidget *w, GdkEventExpose *evt, gpointer user_data)
+{	
+    gdk_window_clear_area(w->window, evt->area.x, evt->area.y,
+            evt->area.width, evt->area.height);
+    
+    return TRUE;
+}
+
+static void
+style_set_cb(GtkWidget *w, GtkStyle *old, gpointer user_data)
+{
+    XfceDesktop *desktop = user_data;
+    
+    if(desktop->priv->bg_pixmap) {
+        gdk_window_set_back_pixmap(w->window, desktop->priv->bg_pixmap, FALSE);
+        gtk_widget_queue_draw(w);
+    }
+}
+
 
 /* public api */
 
@@ -724,6 +721,13 @@ xfce_desktop_new(GdkScreen *gscreen, McsClient *mcs_client)
 	g_signal_connect(G_OBJECT(gscreen), "size-changed",
 			G_CALLBACK(screen_size_changed_cb), desktop);
 	
+	gtk_widget_add_events(GTK_WIDGET(desktop), GDK_EXPOSE);
+	g_signal_connect(G_OBJECT(desktop), "expose-event",
+			G_CALLBACK(desktop_expose_cb), NULL);
+	
+	g_signal_connect(G_OBJECT(desktop), "style-set",
+			G_CALLBACK(style_set_cb), desktop);
+	
 	return GTK_WIDGET(desktop);
 }
 
